      SUBROUTINE NTRAN(IBAND, JMIN, IPOINT, MONONE, MXONE) 
!-----------------------------------------------
!   M o d u l e s 
!-----------------------------------------------
      USE vast_kind_param
      USE params_M 
      USE jmix_dat_M
      USE bands_dat_M
      USE trans_dat_M 
      USE slat_dat_M
      USE vibfn_dat_M
      USE line_dat_M
      USE xsect_dat_M 
      USE solar
      USE molec_dat_M
!...Translated by Pacific-Sierra Research 77to90  4.4D  16:36:33   4/10/03  
!...Switches: -yf12               
!  ---
!
!     COMPUTE MONOCHROMATIC TRANSMITTANCES AT EACH WAVELENGTH
!     FOR BANDPASS IBAND.  TRANMSITTANCES ARE COMPUTED FOR SPECTRA
!     BETWEEN SPECTRUM ISCAN(IBAND,JMIN) AND ISCAN(IBAND,NSCAN(IBAND)).
!
!          IBAND=BAND PASS NUMBER
!          JSCAN=ISCAN(IBAND,J)=SPECTRUM NUMBER FOR CALCULATIONS
!          IPOINT,MONONE-INDICES IN TCALC ARRAY FOR CALCULATED TRANSMITTANCES
!  ---
      IMPLICIT NONE
!-----------------------------------------------
!   G l o b a l   P a r a m e t e r s
!-----------------------------------------------
!-----------------------------------------------
!   D u m m y   A r g u m e n t s
!-----------------------------------------------
      INTEGER :: IBAND 
      INTEGER :: JMIN 
      INTEGER :: IPOINT 
      INTEGER :: MONONE 
      INTEGER :: MXONE 
!-----------------------------------------------
!   L o c a l   P a r a m e t e r s
!-----------------------------------------------
!-----------------------------------------------
!   L o c a l   V a r i a b l e s
!-----------------------------------------------
      INTEGER :: J,  ICINDX,  MSTOR,IR !,, MXMAX, MADD, I 
      INTEGER :: NMON, NSCANS, INDXX, KSMAX2, K, JSCAN, ICINDX2, MADD, I, MXMAX 
      REAL(DOUBLE) :: FACMAS
!-----------------------------------------------

!  --- DECLARE VARIABLES INTEGER*4 TO AVOID OVERFLOWS
 
!  --- NMON=NUMBER OF MONOCHROMATIC POINTS FOR THE BANDPASS CALCULATION
      NMON = NM(IBAND) 
      NSCANS = NSCAN(IBAND) 
!  --- ZERO APPROPRIATE TRANSMISSION ARRAY ELEMENTS FOR CROSS SECTION
!  ---  CALCULATIONS
      MADD = MONONE 
      DO INDXX = 1, NSCANS 
         IF (INDXX >= JMIN) THEN 
            TCALC(IPOINT,MADD:NMON-1+MADD) = 0.D0 
         ENDIF 
         MADD = MADD + NM(IBAND) 
      END DO 
!  --- MAXIMUM LAYER FOR SUMMING CROSS SECTIONS CALCULATIONS
      KSMAX2 = KZTAN(ISCAN(IBAND,NSCANS)) 
!                   ------------LOOP OVER LAYERS
      DO K = 1, KSMAX2 
         MADD = MONONE 
!                   ------------LOOP OVER SPECTRA
         DO INDXX = 1, NSCANS 
            IF (INDXX >= JMIN) THEN 
               JSCAN = ISCAN(IBAND,INDXX) 
               IF (K <= KZTAN(JSCAN)) THEN 
                  FACMAS = CCC(KSCAN(JSCAN),K)/PMASMX(K) 
!                   ------------LOOP OVER FREQUENCIES
                  MXMAX = MXONE + NMON - 1 
                  DO J = 1, NMON 
!                   ------------LOOP OVER RETRIEVAL GASES
                     ICINDX = MXONE + J - 1 
                     MSTOR = MADD + J - 1 
                     ! don't apply shift to first point
                     TCALC(IPOINT,MSTOR) = TCALC(IPOINT,MSTOR) + (X(1,K)/XORG(1,K))*CROSS(&
                           1,K,ICINDX)*FACMAS 
                     IF (IFDIFF/=0) THEN 
                       DO IR = 2, NRET 
!  --- APPLY DIFFERENTIAL WAVENUMBER SHIFT
                           ICINDX2 = ICINDX + ISHIFT(IR-1) 
!  --- FIXUP AT ENDPOINTS
                           ICINDX2 = MAX0(MXONE,ICINDX2) 
                           ICINDX2 = MIN0(MXMAX,ICINDX2) 
                           TCALC(IPOINT,MSTOR) = TCALC(IPOINT,MSTOR) + (X(IR,K)/XORG(IR,K))*CROSS(&
                              IR,K,ICINDX2)*FACMAS 
                       END DO 
!                      ------------BACKGROUND GASES
                       TCALC(IPOINT,MSTOR) = TCALC(IPOINT,MSTOR) + CROSS(NRET+1,K&
                          ,ICINDX2)*FACMAS 
                     ELSE
                       DO IR = 2, NRET 
                          TCALC(IPOINT,MSTOR) = TCALC(IPOINT,MSTOR) + (X(IR,K)/XORG(IR,K))*CROSS(&
                             IR,K,ICINDX)*FACMAS 
                       END DO 
!                      ------------BACKGROUND GASES
                       TCALC(IPOINT,MSTOR) = TCALC(IPOINT,MSTOR) + CROSS(NRET+1,K&
                          ,ICINDX)*FACMAS 
                     ENDIF
                     
                  END DO 
               ENDIF 
            ENDIF 
            MADD = MADD + NM(IBAND) 
         END DO 
      END DO 
!  --- COMPUTE MONOCHROMATIC TRANSMITTANCES FROM CROSS SECTION SUMS
      MADD = MONONE 
      DO INDXX = 1, NSCANS 
         IF (INDXX >= JMIN) THEN 
            DO I = 1, NMON 
               if( ABS( TCALC(IPOINT,MADD+I-1)) .GT. 664.0 ) THEN
                  ! LIMIT SO ONLY GET EXPONENT < 300
                  TCALC(IPOINT,MADD+I-1) = 664.0d0
               ENDIF
               TCALC(IPOINT,MADD+I-1) = EXP((-TCALC(IPOINT,MADD+I-1))) 
            END DO 
         ENDIF 
         MADD = MADD + NM(IBAND) 
      END DO 
      RETURN  
      END SUBROUTINE NTRAN 

      SUBROUTINE GASNTRAN(IR,IBAND, JMIN, IPOINT, MONONE, MXONE) 
!-----------------------------------------------
!   M o d u l e s 
!-----------------------------------------------
      USE vast_kind_param
      USE params_M 
      USE jmix_dat_M
      USE bands_dat_M
      USE trans_dat_M 
      USE slat_dat_M
      USE vibfn_dat_M
      USE line_dat_M
      USE xsect_dat_M 
      USE solar
      USE molec_dat_M
!...Translated by Pacific-Sierra Research 77to90  4.4D  16:36:33   4/10/03  
!...Switches: -yf12               
!  ---
!
!     COMPUTE MONOCHROMATIC TRANSMITTANCES AT EACH WAVELENGTH
!     FOR BANDPASS IBAND FOR ONE GAS.  TRANMSITTANCES ARE COMPUTED FOR SPECTRA
!     BETWEEN SPECTRUM ISCAN(IBAND,JMIN) AND ISCAN(IBAND,NSCAN(IBAND)).
!
!          IR= GAS NUMBER
!          IBAND=BAND PASS NUMBER
!          JSCAN=ISCAN(IBAND,J)=SPECTRUM NUMBER FOR CALCULATIONS
!          IPOINT,MONONE-INDICES IN TCALC ARRAY FOR CALCULATED TRANSMITTANCES
!  ---
      IMPLICIT NONE
!-----------------------------------------------
!   G l o b a l   P a r a m e t e r s
!-----------------------------------------------
!-----------------------------------------------
!   D u m m y   A r g u m e n t s
!-----------------------------------------------
      INTEGER :: IR 
      INTEGER :: IBAND 
      INTEGER :: JMIN 
      INTEGER :: IPOINT 
      INTEGER :: MONONE 
      INTEGER :: MXONE 
!-----------------------------------------------
!   L o c a l   P a r a m e t e r s
!-----------------------------------------------
!-----------------------------------------------
!   L o c a l   V a r i a b l e s
!-----------------------------------------------
      INTEGER :: NMON, NSCANS, KSMAX2, K, JSCAN,ICINDX, MSTOR, &
         MXMAX, J, MADD, I 
      REAL(DOUBLE) :: FACMAS, XFAC 
!-----------------------------------------------

!  --- DECLARE VARIABLES INTEGER*4 TO AVOID OVERFLOWS
 
!  --- NMON=NUMBER OF MONOCHROMATIC POINTS FOR THE BANDPASS CALCULATION
      NMON = NM(IBAND) 
      NSCANS = NSCAN(IBAND) 
!  --- ZERO APPROPRIATE TRANSMISSION ARRAY ELEMENTS FOR CROSS SECTION
!  ---  CALCULATIONS
      MADD = MONONE 
            TCALC(IPOINT,MADD:NMON-1+MADD) = 0.D0 
!  --- MAXIMUM LAYER FOR SUMMING CROSS SECTIONS CALCULATIONS
      KSMAX2 = KZTAN(ISCAN(IBAND,NSCANS)) 
!                   ------------LOOP OVER LAYERS
      DO K = 1, KSMAX2 
         MADD = MONONE 
               JSCAN = ISCAN(IBAND,JMIN) 
               IF (K <= KZTAN(JSCAN)) THEN 
                  FACMAS = CCC(KSCAN(JSCAN),K)/PMASMX(K) 
!                   ------------LOOP OVER FREQUENCIES
                  DO J = 1, NMON 
                        XFAC = X(IR,K)/XORG(IR,K) 
                        ICINDX = MXONE + J - 1 
                        MSTOR = MADD + J - 1 
                        IF (IR/=1 .AND. IFDIFF/=0) THEN 
!  --- APPLY DIFFERENTIAL WAVENUMBER SHIFT
                           ICINDX = ICINDX + ISHIFT(IR-1) 
                           MXMAX = MXONE + NMON - 1 
!  --- FIXUP AT ENDPOINTS
                           ICINDX = MAX0(MXONE,ICINDX) 
                           ICINDX = MIN0(MXMAX,ICINDX) 
                        ENDIF 
                        TCALC(IPOINT,MSTOR) = TCALC(IPOINT,MSTOR) + XFAC*CROSS(&
                           IR,K,ICINDX)*FACMAS 
                     
                  END DO 
               ENDIF 
      END DO 
!  --- COMPUTE MONOCHROMATIC TRANSMITTANCES FROM CROSS SECTION SUMS
      MADD = MONONE 
            DO I = 1, NMON 
               if( ABS( TCALC(IPOINT,MADD+I-1)) .GT. 664.0 ) THEN
                  ! LIMIT SO ONLY GET EXPONENT < 300
                  TCALC(IPOINT,MADD+I-1) = 664.0d0
               ENDIF
               TCALC(IPOINT,MADD+I-1) = EXP((-TCALC(IPOINT,MADD+I-1))) 
            END DO 
        RETURN  
      END SUBROUTINE GASNTRAN 

      SUBROUTINE ZERONTRAN(IBAND, JMIN, IPOINT, MONONE, MXONE) 
!-----------------------------------------------
!   M o d u l e s 
!-----------------------------------------------
      USE vast_kind_param
      USE params_M 
      USE jmix_dat_M
      USE bands_dat_M
      USE trans_dat_M 
      USE slat_dat_M
      USE vibfn_dat_M
      USE line_dat_M
      USE xsect_dat_M 
      USE solar
      USE molec_dat_M
!...Translated by Pacific-Sierra Research 77to90  4.4D  16:36:33   4/10/03  
!...Switches: -yf12               
!  ---
!
!     COMPUTE MONOCHROMATIC TRANSMITTANCES AT EACH WAVELENGTH
!     FOR BANDPASS IBAND SET TO 1.0.  TRANMSITTANCES ARE COMPUTED FOR SPECTRA
!     BETWEEN SPECTRUM ISCAN(IBAND,JMIN) AND ISCAN(IBAND,NSCAN(IBAND)).
!
!          IBAND=BAND PASS NUMBER
!          JSCAN=ISCAN(IBAND,J)=SPECTRUM NUMBER FOR CALCULATIONS
!          IPOINT,MONONE-INDICES IN TCALC ARRAY FOR CALCULATED TRANSMITTANCES
!  ---
      IMPLICIT NONE
!-----------------------------------------------
!   G l o b a l   P a r a m e t e r s
!-----------------------------------------------
!-----------------------------------------------
!   D u m m y   A r g u m e n t s
!-----------------------------------------------
      !INTEGER :: IR 
      INTEGER :: IBAND 
      INTEGER :: JMIN 
      INTEGER :: IPOINT 
      INTEGER :: MONONE 
      INTEGER :: MXONE 
!-----------------------------------------------
!   L o c a l   P a r a m e t e r s
!-----------------------------------------------
!-----------------------------------------------
!   L o c a l   V a r i a b l e s
!-----------------------------------------------
      INTEGER :: NMON, MADD, I 
      !REAL(DOUBLE) :: FACMAS 
!-----------------------------------------------

!  --- DECLARE VARIABLES INTEGER*4 TO AVOID OVERFLOWS
 
!  --- NMON=NUMBER OF MONOCHROMATIC POINTS FOR THE BANDPASS CALCULATION
      NMON = NM(IBAND) 
      !NSCANS = NSCAN(IBAND) 
!  --- ZERO APPROPRIATE TRANSMISSION ARRAY ELEMENTS FOR CROSS SECTION
!  ---  CALCULATIONS
      MADD = MONONE 
            TCALC(IPOINT,MADD:NMON-1+MADD) = 0.D0 
!  --- COMPUTE MONOCHROMATIC TRANSMITTANCES FROM CROSS SECTION SUMS
      MADD = MONONE 
            DO I = 1, NMON 
               if( ABS( TCALC(IPOINT,MADD+I-1)) .GT. 664.0 ) THEN
                  ! LIMIT SO ONLY GET EXPONENT < 300
                  TCALC(IPOINT,MADD+I-1) = 664.0d0
               ENDIF
               TCALC(IPOINT,MADD+I-1) = EXP((-TCALC(IPOINT,MADD+I-1))) 
            END DO 
      RETURN  
      END SUBROUTINE ZERONTRAN 
