module solar

! Oct 2005
! Added code so we are equivalent to Hase solar 4.3
! Aug 2005
! Added code so we are equivalent to Hase solar 4.2
!
! July 2005
! Added ABS() to active window calculation so the solar emission line
!   at 811.57 will be seen.
!
! January 2005
! added FP underflow check line 490
!
! September 2004
!  - Moved parameter module globsol3 into this module and added CONTAIN
!     statement
!  - Moved contents of solar_dat_M to this file
!  - Update calculation to FH ver4 changed subroutine name to solarfh
!
!     Last change:  BLA  28 May 2004    3:17 pm
!
!
! Model of the infrared solar transmittance spectrum
!
! by
!
! F. Hase(1), P. Demoulin(2), A. Goldman(3), J. W. Hannigan(4),
! C. P. Rinsland(5), and G. C. Toon(6)
!     1: IMK, Forschungszentrum und Universitaet Karlsruhe, Germany,
!        <frank.hase@imk.fzk.de>
!     2: Institute de Astrophysique, Univ. Liege, Belgium
!     3: Department of Physics, University of Denver, Denver, CO, USA
!     4: National Center for Atmospheric Research, Boulder, CO, USA
!     5: NASA Langley Research Center, Hampton, VA
!     6: Jet Propulsion Laboratory, Pasadena, CA, USA

USE vast_kind_param
USE params_M

implicit none

REAL(DOUBLE), DIMENSION(:), ALLOCATABLE :: TCO
REAL(DOUBLE), DIMENSION(LNMXCO) :: WSOLAR
REAL(DOUBLE), DIMENSION(LNMXCO) :: ADOP
REAL(DOUBLE), DIMENSION(LNMXCO) :: STR
REAL(DOUBLE), DIMENSION(5) :: CPARM
REAL(DOUBLE), DIMENSION(5) :: CIPARM
INTEGER, DIMENSION(5) :: ICFIX
INTEGER :: NCOLNS, IFCO, NSOLAR, NSOLAR1, NSOLAR2
REAL(DOUBLE) :: TEMPCO, TREFCO, FOVDIA


character*200 :: lindatei
integer :: nptsmw

real(8) :: firstnue, dnue, semiextfov, longrad, latrad, juld, fovratio
real(8) :: nueskalcenter, phisol, nueskal_inp
logical :: dec_doppler

type liniendaten
    sequence
    real(8) :: nue
    real(8) :: strength
    real(8) :: gausswidth
    real(8) :: sklor
    real(8) :: variStrength
    real(8) :: variWidth
    integer :: ileft,iright
end type

CONTAINS

subroutine solarfh( flag )

! CHANGES
!
! 7 JUN 2004
!  - converted to subroutine to sfit2.
!  - the plan is to loop this program (and all its subs) over NBAND and store
!    the transmission spectra consecutivly in TCO.
!  - added fit of shift parameter

USE vast_kind_param
USE bands_dat_M
USE trans_dat_M
USE jmix_dat_M
USE tfiles_dat_M

implicit none

logical :: isfile
integer :: i,nfovmax !,zaehler
integer :: iband, kband, mxlnperbnd, flag
integer :: year, month, day, hour, minute
real(8) :: secs, lat, lon

integer, dimension(MAXBND),save :: nlnread
integer, dimension(MAXBND) :: nlnfst, nlnlst
real(8) :: nueskal, radius, xpos, zpos, xznorm, weight, sumweight
real(8), dimension(:), allocatable :: soltrm_mean, soltrm

type(liniendaten),dimension(:,:), pointer, save :: lines
!type(liniendaten),dimension(:,:), allocatable, save :: lines

dec_doppler = .FALSE.
nueskal_inp = 1.0d0;

!  --- Clean up and exit
if( flag == 2 ) then
    if( associated(lines)) deallocate (lines)
   goto 100
   endif

! INITIALIZE SOLAR X-SECTION ARRAY
TCO(:NCROSS) = 0.D0

lindatei = TFIL11          ! use old line list filespec
dnue = dn                  ! monochromatic point spacing
semiextfov = fovdia/2000.  ! FOV half radius in radians

! read data from input file
!print *,'Reading input file ...'
! call ReadInput('solar3.inp')
!print *,'... done!'

! do a quick trip through the solar line file and determine the # of lines
! in each band

if ( flag == 0 ) then

!  --- sfit2 has no accomodation for lat, lon, date/time
!  --- check for file obs.dat - observation data input file
   inquire(file='obs.dat', exist=isfile)
   if( .not. isfile ) then
!  --- use some default lat, lon, date
          juld    = 2452000.0d0
          latrad  = 0.0d0
          longrad = 0.0d0
          write(16,*)'SOLARFH - Setting default lat, lon, date'
   else
      open(unit=21, file='obs.dat', status='old')
          read(21,*) year, month, day, hour, minute, secs
          read(21,*) lat, lon
          close(21)
          juld    = juldat( year, month, day, hour, minute, secs )
          latrad  = lat * PI /180.0d0
          longrad = lon * PI /180.0d0
        !print *, year, month, day, hour, minute, secs, lat, lon
          !print *, juld, latrad, longrad
   endif


! determine relevant astronomical values
!    nueskalcenter: scaling of frequency axis due to Earth's orbital motion and rotation
!                   on center of solar disk
!    fovratio: ratio FOV diam./solar diam.
!    phisol: angle of solar axis towards earth
   print *,'    Calculation of astronomical quantities ...'
   call ephemerid(juld,latrad,longrad,semiextfov,nueskalcenter, &
                  fovratio,phisol)
   WRITE(*,101)'      radial velocity (pos. receding) [km/s]:  ',2.99792d5 * (1.0d0 - nueskalcenter)
   WRITE(*,101)'      scaling factor due to Doppler effect:    ',nueskalcenter
   WRITE(*,101)'      FOV in relative units of solar diameter: ',fovratio
   WRITE(*,101)'      inclination of solar axis [deg.]:        ', 57.2958 * phisol

101 FORMAT( A, ES21.6 )

!  --- for rest system
   if (dec_doppler) nueskalcenter = nueskal_inp

      ncolns = 0
      mxlnperbnd = 0

      do iband = 1, nband
         firstnue = wmon(iband)                 ! includes 10 reso units for shifts
      nptsmw   = nm(iband)
         call LookAtLines( nlnfst(iband),nlnlst(iband) )
      nlnread(iband) = nlnlst(iband) - nlnfst(iband) + 1    ! line in this band
      print *,'    Band: ', iband, 'Number of lines read + alloc: ',nlnread(iband)
      ncolns = ncolns + nlnread(iband)          ! sum of all solar lines
         mxlnperbnd = max( mxlnperbnd, nlnread(iband) )
      end do

      !print *,'    Max Number of lines per band: ',mxlnperbnd
      if ( ncolns == 0 ) goto 100
      allocate( lines( 1:mxlnperbnd, 1:nband ))
endif

!print *, '     size lines', size(lines)
!print *, '    someline   ', lines(1,1)%nue
kband = 1
bands: do iband = 1, nband

      !print *, '    Solar spectrum calculation band: ',iband
   firstnue = wmon(iband)
   nptsmw   = nm(iband)

!  --- Initialize spectral arrays
! (in soltrm the spectrum for a certain position on the solar disk will be written)
! (in soltrm_mean the resulting solar spectrum for given FOV will be written)
   !print *,'Allocating spectral arrays ...'
   allocate (soltrm_mean(1:nptsmw),soltrm(1:nptsmw))
   soltrm_mean = 0.0d0
   soltrm = 0.0d0

   if ( flag == 0 ) then
      call ReadLines( nlnfst(iband), nlnlst(iband), lines( :, iband ))
      call LineBounds( nlnfst(iband), nlnlst(iband), nueskalcenter, &
                                lines( :, iband ))
    endif

!  --- FOV integration
! nfovmax determines FOV gridpoints: -nfovmax..+nfovmax x -nfovmax..+nfovmax
! xpos,zpos: determine position on solar disk in relative coordinates (right
! handed coordinate system, z-direction perpendicular to observer's LOS, aligned
! with projection of solar rotation axis on celestial sphere, y-axis along LOS



!print *,'      Performing FOV-integration ...'
!  --- Simplified FOV integration, assumes inclination of sun's equator to be zero

   !zaehler = 0
   nfovmax = nint(20.0d0 * fovratio)
   xznorm = fovratio / (nfovmax + 0.5d0)
        sumweight = 0.0d0
        do i = -nfovmax,nfovmax
        xpos = i * xznorm
        zpos = 0.5d0 * xznorm * sqrt(real(nfovmax * nfovmax - i * i,8))
        radius = sqrt(xpos * xpos + zpos * zpos)
        nueskal = nueskalcenter * NueSkalDisk(xpos,zpos,0.0d0)
!        nueskal = nueskalcenter * NueSkalDisk(xpos,zpos,phisol)
        call MakeSolspec( radius,nueskal,nlnread(iband),lines( :, iband), &
                         soltrm )
        weight = sqrt(1.0d0 - xpos * xpos)
        soltrm_mean = soltrm_mean + weight * soltrm
        sumweight = sumweight + weight
   end do

    soltrm_mean = soltrm_mean / sumweight

   ! Spektrum rausschreiben
   !print *,'  Writing spectrum to file ...'
   call tofile_spec('solspec.dat',nptsmw,firstnue,dnue,soltrm_mean)

!print *, 'size soltrm', size(soltrm_mean)
!print *, kband, nm(iband), kband, nm(iband)+kband-1

    TCO( kband : kband + nm(iband) - 1) = soltrm_mean
    kband = kband + nm(iband)

    !print *, '     size lines', size(lines)
   ! Deallocierung der Felder
   deallocate (soltrm_mean,soltrm)

end do bands

if( flag == 0 ) print *,' Solar spectrum calculation completed.'

100 continue

end subroutine solarfh


!====================================================================
!  ephemerid: calculates Doppler scaling of solar spectrum due to
!                 orbital motion and rotation of Earth
!====================================================================
subroutine ephemerid(juld,latrad,longrad,semiextfov,nueskalcenter,fovratio,phisol)

implicit none

real(8),intent(in) :: juld,latrad,longrad,semiextfov
real(8),intent(out) :: nueskalcenter,fovratio,phisol

real(8),parameter :: pi = 3.141592653589793d0
real(8) :: zeit,anom,danom_dt,center,dcenter_dt,vrad_orbit,vrad_diurn &
  ,sideral,laen_true,alpha,delta,kwert,lnull

! Zeit vs. 1.Jan. 2000 1.5 TD in Jul. Jahrhunderten
zeit = (juld - 2.451545d6) / 3.6525d4

!====================================================================
! Berechnung der Orbitalkomponente der Radialgeschwindigkeit

! Mittlere Anomalie in rad (und zeitl. Ableitung der m. A. in rad/s)
anom = 6.2400600d0 + zeit * (628.3019553d0 - zeit * (2.72100d-6 &
  + 8.38d-9 * zeit))
danom_dt = 1.99097d-7

! Mittelpunktsgleichung
center = (3.3416073d-2 - zeit * (8.4073d-5 + 2.44d-7 * zeit)) &
  * sin(anom) + (3.48944d-4 - 1.763d-6 * zeit) * sin(2.0d0 * anom) &
  + 5.061d-6 * sin(3.0d0 * anom)
dcenter_dt = danom_dt * (3.3416d-2 * cos(anom) + 3.489d-4 * cos(2.0d0 * anom))

! Radialgeschwindigkeit durch Erdbewegung (astr. Konvention!), m/s
vrad_orbit = 2.50d9 * (danom_dt + dcenter_dt) * sin(anom + center)

!====================================================================
! Berechnung der taeglichen Komponente der Radialgeschwindigkeit

! Mittlere Sternzeit Greenwich (bezogen auf mittl Frhlingspunkt) in rad
sideral = modulo(4.894961212d0 + 6.30038809781d0 * (juld - 2.451545d6) &
+ 6.770708d0 * zeit * zeit - 4.5087d-10 * zeit * zeit * zeit,2.0d0 * pi)

! wahre Laenge der Sonne in rad (Summe aus center und mittl. Laenge)
laen_true = center + 4.89506300d0 + 6.283319668d2 * zeit + 5.2918d-6 * zeit * zeit

! Rektaszension und Deklination der Sonne in rad
alpha = atan2(0.917523d0 * sin(laen_true),cos(laen_true))
delta = asin(0.39768d0 * sin(laen_true))

! Radialgeschwindigkeit durch Erddrehung (astr. Konvention!), m/s
vrad_diurn = 4.63d2 * cos(latrad) * cos(delta) * sin(sideral - longrad - alpha)

!====================================================================
! Berechnung des resultierenden Skalierungsfaktors

nueskalcenter = 1.0d0 - 3.33564d-9 * (vrad_diurn + vrad_orbit)

!====================================================================
! Berechnung des relativen Gesichtsfeldes

fovratio = 2.1486d2 * semiextfov * (1.0d0 + 1.671d-2 * cos(anom + center))
if (fovratio .gt. 1.0d0) fovratio = 1.0d0

!====================================================================
! Berechnung der Neigung des Sonnenaequators (genaehert)

! geometrische mittlere Laenge der Sonne
lnull = 4.8950630d0 + 6.283319668d2 * zeit - 5.2918d-6 * zeit * zeit
kwert = 1.28573d0 + 6.6699211d-7 * (juld - 2396758.0d0)
phisol = 0.1265d0 * sin(lnull + center - kwert)

end subroutine ephemerid



!====================================================================
!  gonext: Einlesen bis zum naechsten $ Zeichen
!====================================================================
subroutine gonext(ifile)

implicit none
integer,intent(in) :: ifile
character(1) :: nextchar

nextchar='x'
do while (nextchar /= '$')
    read(ifile,'(a1)') nextchar
end do

end subroutine gonext



!====================================================================
!  juldat: calculates Julian Date
!====================================================================
real(8) function juldat (jahr_in,monat_in,tag,stunde,minute,sekunde)

implicit none

integer,intent(in) :: jahr_in,monat_in,tag,stunde,minute
real(8),intent(in) :: sekunde

integer :: jahr,monat,iwerta,iwertb
real(8) :: ftag

ftag = tag + stunde / 24.0d0 + minute / (1.440d3) + sekunde / (8.6400d4)

if (monat_in .le. 2) then
    jahr = jahr_in - 1
    monat = monat_in + 12
else
    jahr = jahr_in
    monat = monat_in
end if

iwerta = int(0.01 * jahr)
iwertb = 2 - iwerta + int(0.25d0 * iwerta)

juldat = int(3.6525d2 * (jahr + 4716)) + int(3.06001d1 * (monat + 1)) &
 + ftag + iwertb - 1.5245d3

end function juldat



!====================================================================
!  LineBounds: determines actve window for line calculation
!====================================================================
subroutine LineBounds(nfirstline,nlastline,nueskal,lines)


implicit none

integer,intent(in) :: nfirstline,nlastline
type(liniendaten),dimension(nlastline-nfirstline+1),intent(inout) :: lines
real(8),intent(in) :: nueskal

integer :: i,nfirst,nlast
real(8) :: nueradius

do i = 1,nlastline-nfirstline+1
    nueradius = 4.0d0 * (abs(lines(i)%strength) + 0.25d0) &
      * (9.0d0 * lines(i)%gausswidth) * (1.0d0 + 5.0d0 * abs(lines(i)%sklor))
    nfirst = nint((lines(i)%nue * nueskal - firstnue - nueradius) / dnue)
    nlast  = nint((lines(i)%nue * nueskal - firstnue + nueradius) / dnue)
    lines(i)%ileft = nfirst
    lines(i)%iright = nlast
    if (lines(i)%ileft .lt. 1) then
        lines(i)%ileft = 1
    else
        if (lines(i)%ileft > nptsmw) lines(i)%ileft = nptsmw
    end if
    if (lines(i)%iright > nptsmw) then
        lines(i)%iright = nptsmw
    else
        if (lines(i)%iright < 1) lines(i)%iright = 1
    end if
end do

end subroutine LineBounds



!====================================================================
!  LookAtLines: determines number of lines in database
!====================================================================
subroutine LookAtLines(nfirstline,nlastline)


implicit none

integer,intent(out) :: nfirstline,nlastline

!character(len=85) :: dumchar
integer :: zaehler,status
real(8),parameter :: nueextradius = 20.0d0  ! Erweiterungsbereich fuer Linienauswahl
real(8) :: nue

open (10,file=trim(lindatei), iostat = status, status='old',action='read')
if (status /= 0) print *,'Cannot find solar linedata file'

call gonext(10)

zaehler = 0
outer: do
    read (10,'(F12.6,A76)',end = 100,iostat = status) nue !,dumchar
    if (status /= 0) print *,'Sub lookatlines: Cannot read solar lines!'
        zaehler = zaehler + 1
        if (firstnue - nueextradius .lt. nue) then
            nfirstline = zaehler
            do
                read (10,'(F12.6,A76)',end = 100,iostat = status) nue !,dumchar
                if (status /= 0) print *,'Sub lookatlines: Cannot read solar lines!'
                zaehler = zaehler + 1
                if (firstnue + dnue * (nptsmw-1) + nueextradius .lt. nue) then
                    zaehler = zaehler - 1
                    exit outer
                end if
            end do
        end if
end do outer
100 continue
 close (10)
nlastline = zaehler

end subroutine LookAtLines



!====================================================================
!  MakeSolspec: calculates solar spectrum for specified position
!                on solar disk
!====================================================================
subroutine MakeSolspec(radius,nueskal,noflines,lines,soltrm)

!USE solar_dat_M

implicit none

integer,intent(in) :: noflines
real(8),intent(in) :: radius,nueskal
type(liniendaten),dimension(noflines),intent(in) :: lines

real(8),dimension(nptsmw),intent(out) :: soltrm

integer :: i,j
real(8) :: linstrength,width,widthqu,widthkub,widthquad,deltanue,restwert
real(8) :: wshift,nue,tup,tlow, exarg

! decode fit parameters
wshift = CPARM(4) - 1.D0      !shift

soltrm = 0.0d0

! Gesamte optische Dicke aus Beitraegen aller Linien bestimmen
do i = 1,noflines
    if (lines(i)%iright - lines(i)%ileft .gt. 0) then
        linstrength = lines(i)%strength * (1.0d0 + lines(i)%variStrength * radius * radius)
        width = lines(i)%gausswidth * (1.0d0 + lines(i)%variWidth * radius * radius)
        widthqu = width * width
        widthkub = widthqu * width
        widthquad =  widthkub * width
        do j = lines(i)%ileft,lines(i)%iright
            deltanue = abs(firstnue + dnue * (j-1) - nueskal * ( lines(i)%nue + wshift ))
            exarg = deltanue * deltanue / sqrt(widthquad + abs(lines(i)%sklor) * (-0.54d0 * widthquad + deltanue *  &
                    (0.33d0 * widthkub + deltanue * (0.12d0 * widthqu + 0.342d0 * deltanue * width))))
            exarg = MIN(exarg, 664.0d0)
            soltrm(j) = soltrm(j) + linstrength * exp( -exarg )
        end do
    end if
end do

! Uebergang auf Transmissionsspektrum, Minnaert-Offset
! Minnaert-Offset for extended wave number range (valid from 700 cm-1 to 25000 cm-1)

do i = 1,nptsmw
    nue = firstnue + dnue * (i - 1)
    tup = 5700.0d0 + 0.28d0 * (nue - 2000.0d0)
    tlow = 4000.0d0
    restwert =  (exp(1.4387d0 * nue / tup) - 1.0d0) / (exp(1.4387d0 * nue / tlow) - 1.0d0)
    soltrm(i) = restwert + (1.0d0 - restwert) * exp(-soltrm(i))
end do

end subroutine MakeSolspec



!====================================================================
!  NueSkalDisk:
!====================================================================
real(8) function NueSkalDisk(xpos,zpos,phisol)

implicit none

real(8),intent(in) :: xpos,zpos,phisol

real(8),parameter :: pi = 3.141592653589793d0
real(8) :: latsol,omega

if (xpos * xpos + zpos * zpos .gt. 1.0d0) then
    NueSkalDisk = 1.0d0
else
    ! determine heliographic latitude associated with xpos,zpos:
    latsol = 0.5d0 * pi - acos(zpos * cos(phisol) - sin(phisol) * &
      sqrt(1.0d0 - xpos * xpos - zpos * zpos))

    ! determine solar rotation frequency to this latitude [rad/s]
    omega = 2.0d0 * pi * (4.2711d-7 + &
      (-9.599d-8 + 1.276d-8 * latsol * latsol) * latsol * latsol)

    ! determine scaling factor
    NueSkalDisk = (1.0d0 - 2.3225d0 * omega * cos(phisol) * xpos)
end if

end function NueSkalDisk



!====================================================================
!  ReadInput: Einlesen der Linien
!====================================================================
subroutine ReadInput(dateiname)


implicit none

character(len=*),intent(in) :: dateiname

open (10,file = trim(dateiname),status = 'old')
call gonext(10)
read (10,*) lindatei

call gonext(10)
read (10,*) firstnue
read (10,*) dnue
read (10,*) nptsmw

call gonext(10)
read (10,*) juld
read (10,*) latrad
read (10,*) longrad
read (10,*) dec_doppler
if (dec_doppler) read (10,*) nueskal_inp

call gonext(10)
read (10,*) semiextfov

close (10)

end subroutine ReadInput



!====================================================================
!  Read solar lines
!====================================================================
subroutine ReadLines(nfirstline,nlastline,lines)


implicit none

integer,intent(in) :: nfirstline,nlastline
type(liniendaten),dimension(1:nlastline-nfirstline+1),intent(inout) :: lines

integer :: i,status
!real(8) :: dumreal

open (10,file=trim(lindatei), iostat = status, status='old',action='read')
if (status /= 0) print *,'Sub readsolarlines: Error in opening file'

do i = 1,nfirstline-1
    read (10,*)
end do

do i = 1, nlastline - nfirstline + 1
    read (10,'(F12.6,ES11.3,ES11.3,ES11.3,ES11.3,ES11.3)',iostat = status) &
       & lines(i)%nue,lines(i)%strength,lines(i)%gausswidth, &
       & lines(i)%sklor,lines(i)%variStrength,lines(i)%variWidth
    if (status /= 0) print *,'Sub readsolarlines: Cannot read solar line!'
end do
 close(10)

end subroutine readlines



!====================================================================
!  tofile_specs: Rausschreiben des Spektrums (Fuer Test-Output)
!====================================================================
subroutine tofile_spec(dateiname,npts,firstnue,dnue,spec)

implicit none

character(*),intent(in) :: dateiname
integer,intent(in) :: npts
real(8),intent(in) :: firstnue,dnue
real(8),dimension(npts),intent(in) :: spec

integer :: i

open (10,file = trim(dateiname),status = 'replace')
    do i = 1,npts
        write (10,'(F12.6,1X,F7.4)') firstnue + dnue * (i - 1), spec(i)
    end do
close (10)

end subroutine tofile_spec


end module solar

